@@ -3,11 +3,12 @@ from scipy.signal import savgol_filter
 import plotly.graph_objects as go
 import numpy as np
 import pysam
-# import pyBigWig
 import plotly.graph_objects as go
 import plotly.io as pio
 import collections
 import pandas as pd
+import io
+from PyPDF2 import PdfFileMerger
 from modbampy import *
 import plotly.express as px
 from plotly.subplots import make_subplots
@@ -58,15 +59,6 @@ def calc_freq(dict_per_read_mod, start,end):
     freq_smooth["x"] = freq["x"]
     return freq, freq_smooth
 
-def calc_rows_h(dicts):
-    num_reads = []
-    for read_dict in dicts:
-        num_reads.append(len(read_dict))
-
-    row_h = list(0.85 * np.array(num_reads)/sum(num_reads))
-
-    return row_h
-
 def queue_reads_plotly(dict_per_read_mod):
     
     sorted_mod = dict(sorted(dict_per_read_mod.items(), key=lambda e: e[1][0], reverse=True))
@@ -97,13 +89,22 @@ def queue_reads_plotly(dict_per_read_mod):
 
     return plot_dict
 
+def pdf_report(figs, output):
+    merger = PdfFileMerger()
+    for fig in figs:
+        pdf_file = io.BytesIO()
+        pio.write_image(fig, pdf_file, 'pdf')
+        pdf_file.seek(0)
+        merger.append(pdf_file)
+        
+    merger.write(output)
 def SetColor(x):
     if(x == 0):
         return "blue"
-#         return "#007da3"
+        # return 'white'
     if(x == 1):
         return "red"
-#         return "#DD4765"
+        # return 'black'
 
 def process_bam(bam, chrom, start, end,tag_name=None, tag_value =None,min_prob=0.5, max_prob=0.5):
 #     tag_name = "HP"
@@ -129,6 +130,34 @@ def process_bam(bam, chrom, start, end,tag_name=None, tag_value =None,min_prob=0
                 
     return dict_per_read_mod
 
+def process_bam_strands(bam, chrom, start, end,tag_name=None, tag_value =None,min_prob=0.5, max_prob=0.5):
+
+    dict_per_read_mod_pos = {}
+    dict_per_read_mod_neg = {}
+    with ModBam(bam) as b:
+            for read in b.reads(chrom, start, end, tag_name=tag_name,tag_value=tag_value):
+                mapped_modbase_pos = {}
+                mapped_modbase_neg = {}
+                read_start = max([read.reference_start,start])
+                read_end = min([read.reference_end,end])
+                read_len = read_end - read_start
+                for pos_mod in read.mod_sites:
+                    qname, rpos, qpos, strand, mod_strand, cbase, mbase, score = pos_mod
+                    if strand == "-":
+                        call = binerize_mod_call(score,min_prob,max_prob)
+                        if call != -1:
+                            mapped_modbase_neg[rpos - 1] = call
+                    else:
+                        call = binerize_mod_call(score,min_prob,max_prob)
+                        if call != -1:
+                            mapped_modbase_pos[rpos] = call
+                if strand == "-":
+                    dict_per_read_mod_neg[read.query_name] = [read_len,(read_start,read_end),collections.OrderedDict(sorted(mapped_modbase_neg.items()))]
+                else:
+                    dict_per_read_mod_pos[read.query_name] = [read_len,(read_start,read_end),collections.OrderedDict(sorted(mapped_modbase_pos.items()))]
+                
+    return dict_per_read_mod_neg, dict_per_read_mod_pos
+
 def get_reads(bams, chrom, start, end, hap=None, strand=None, samp_names = None, min_prob=0.5, max_prob=0.5):
 
     dicts = []
@@ -137,14 +166,14 @@ def get_reads(bams, chrom, start, end, hap=None, strand=None, samp_names = None,
             titles = []
             samp_list = ["Sample " + str(i) for i in range(1, len(bams) + 1)]
             for samp in samp_list:
-                titles.append(samp + " haplotype 1")
-                titles.append(samp + " haplotype 2")
+                titles.append(samp + " (haplotype 1)")
+                titles.append(samp + " (haplotype 2)")
         if samp_names:
             titles = []
             samp_list = samp_names
             for samp in samp_list:
-                titles.append(samp + " haplotype 1")
-                titles.append(samp + " haplotype 2")
+                titles.append(samp + " (haplotype 1)")
+                titles.append(samp + " (haplotype 2)")
 
         for bam in bams:
             reads_hp1 = process_bam(bam,chrom,start,end,tag_name="HP", tag_value=1,min_prob=min_prob,max_prob=max_prob)
@@ -152,6 +181,25 @@ def get_reads(bams, chrom, start, end, hap=None, strand=None, samp_names = None,
             dicts.append(reads_hp1)
             dicts.append(reads_hp2)
 
+    elif strand:
+        if not samp_names:
+            titles = []
+            samp_list = ["Sample " + str(i) for i in range(1, len(bams) + 1)]
+            for samp in samp_list:
+                titles.append(samp + " (strand -)")
+                titles.append(samp + " (strand +)")
+        if samp_names:
+            titles = []
+            samp_list = samp_names
+            for samp in samp_list:
+                titles.append(samp + " (strand -)")
+                titles.append(samp + " (strand +)")
+
+        for bam in bams:
+            reads_neg, reads_pos = process_bam_strands(bam,chrom,start,end,min_prob=min_prob,max_prob=max_prob)
+            dicts.append(reads_neg)
+            dicts.append(reads_pos)
+
 
 
     else:
@@ -165,3 +213,4 @@ def get_reads(bams, chrom, start, end, hap=None, strand=None, samp_names = None,
             dicts.append(reads)
 
     return dicts, titles
+
